function [V_best,cost_best,G_best,n_tot] = mpfa(n,np,nmax,alpha,...
    rdm,speed,betamin,gamma,m,c1,c2,f_opt_min,f_opt_max,teta_min,...
    teta_max,b_min,b_max,h_min,h_max,...
    phi_py_min,phi_py_max,n_py_min,n_py_max,...
    phi_y_min,phi_y_max,n_y_min,n_y_max,...
    phi_px_min,phi_px_max,n_px_min,n_px_max,...
    phi_x_min,phi_x_max,n_x_min,n_x_max,...
    steelcost,concretecost,ksh,shutteringcost,fc,fy,nd,mdx,mdy,dpx,dpy,...
    design_code,spiral,lsrd,gsrd,arm_min,duct,sim_x,sim_y,sim_phi,info)
% Modified Parallel Firefly Algorithm for Design of RC Sections
%
% Input Parameters:
%    n              (scalar)    size of population
%    np             (scalar)    number of subpopulations
%    nmax           (scalar)    maximum number of iterations
%    alpha          (scalar)    randomness in the forward step
%                               [0 none or minimum, ~0.01*n maximum]
%    rdm            (scalar)    value, included in the interval [0,1], 
%                               that indicates when the alpha
%                               parameter begins to be reduced in
%                               the process
%    speed          (logical)   speed in the reduction of the alpha
%                               parameter [0 slow, 1 fast]
%    betamin        (scalar)    value, within the interval [0,1], of 
%                               the minimum amplitude of the forward step
%    gamma          (scalar)    transparency of the medium ( >= 0 )
%                               (0 = full transparency)
%    m              (scalar)    exponent of r (distance between fireflies)
%                               (m > 1)
%    c1             (scalar)    coefficient, within the interval [1,2],
%                               for 2nd strength constraint
%    c2             (scalar)    coefficient, within the interval [5,50],
%                               for 2nd strength constraint
%    f_opt_min      (scalar)    lower bound, within the interval (-1,0],
%                               for 1st optimization variable
%    f_opt_max      (scalar)    upper bound, within the interval [0,1),
%                               for 1st optimization variable
%    teta_min       (scalar)    lower bound for 2nd design variable (rad)
%    teta_max       (scalar)    upper bound for 2nd design variable (rad)
%    b_min          (scalar)    lower bound for 3rd design variable (mm)
%    b_max          (scalar)    upper bound for 3rd design variable (mm)
%    h_min          (scalar)    lower bound for 4th design variable (mm)
%    h_max          (scalar)    upper bound for 4th design variable (mm)
%    phi_py_min     (scalar)    lower bound for 5th design variable (mm)
%    phi_py_max     (scalar)    upper bound for 5th design variable (mm)
%    n_py_min       (scalar)    lower bound for 6th design variable
%    n_py_max       (scalar)    upper bound for 6th design variable
%    phi_y_min      (scalar)    lower bound for 7th design variable (mm)
%    phi_y_max      (scalar)    upper bound for 7th design variable (mm)
%    n_y_min        (scalar)    lower bound for 8th design variable
%    n_y_max        (scalar)    upper bound for 8th design variable
%    phi_px_min     (scalar)    lower bound for 9th design variable (mm)
%    phi_px_max     (scalar)    upper bound for 9th design variable (mm)
%    n_px_min       (scalar)    lower bound for 10th design variable
%    n_px_max       (scalar)    upper bound for 10th design variable
%    phi_x_min      (scalar)    lower bound for 11th design variable (mm)
%    phi_x_max      (scalar)    upper bound for 11th design variable (mm)
%    n_x_min        (scalar)    lower bound for 12th design variable
%    n_x_max        (scalar)    upper bound for 12th design variable
%    steelcost      (scalar)    price of steel (Euro/mm2/m)
%    concretecost   (scalar)    price of concrete (Euro/mm2/m)
%    ksh            (scalar)    parameter of geometry of formwork
%                               (1 for beams or 2 for columns)
%    shutteringcost (scalar)    price of formwork (Euro/mm/m)
%    fc             (scalar)    specified/design strength of concrete(MPa)
%    fy             (scalar)    specified/design strength of steel (MPa)
%    nd             (scalar)    factored/design axial force (kN)
%    mdx            (scalar)    factored/design flexural moment 
%                               about x axis (kN x m)
%    mdy            (scalar)    factored/design flexural moment
%                               about y axis (kN x m)
%    dpx            (scalar)    clear cover of reinforcement + stirrup
%                               diameter in x direction (mm)
%    dpy            (scalar)    clear cover of reinforcement + stirrup
%                               diameter in y direction (mm)
%    design_code    (scalar)    EC2 = 1;  ACI318 = 2
%    spiral         (logical)   spiral = 1 enables spiral reinforcement
%    lsrd           (logical)   lsrd = 1 enables local perturbation
%    gsrd           (logical)   gsrd = 1 enables global perturbation
%    arm_min        (logical)   arm_min = 1 enables minimum reinforcement
%                               constraints
%    duct           (logical)   duct = 1 enables ductility constraints
%    sim_x          (logical)   sim_x = 1 means phi_px=phi_x and n_px=n_x
%    sim_y          (logical)   sim_y = 1 means phi_py=phi_y and n_py=n_y
%    sim_phi        (logical)   sim_phi = 1 means phi_px=phi_x=phi_py=phi_y
%    info           (logical)   info = 1 enables graphic information
%
% Output Parameters:
%    V_best    (vector)    optimum values of the variables
%    cost_best (scalar)    optimum value of the objective function (Cost)
%    G_best    (vector)    optimum values of constraints (14)
%    n_tot     (scalar)    number of evaluations of the objective function
%
% Nomenclature for arrays, vectors and scalars:
%    XYZ ---> array  XYZ
%    Xyz ---> vector Xyz
%    xyz ---> scalar xyz
%
% Functions:
%    mpfa_cnstr.m
%
% Simetry:
%   CASE 1 (7 variables): sim_x = 1 & sim_y = 1 & sim_phi = 1
%     1   2    3  4    5     6     7 
%     f  teta  b  h   phi   n_y   n_x
%
%   CASE 2 (8 variables): sim_x = 1 & sim_y = 1 & sim_phi = 0
%     1   2    3  4     5      6      7      8
%     f  teta  b  h   phi_y   n_y   phi_x   n_x
%
%   CASE 3 (12 variables): sim_x = 0 & sim_y = 0 & sim_phi = 0
%     1   2    3  4    5      6      7     8     9      10    11     12
%     f  teta  b  h  phi_py  n_py  phi_y  n_y  phi_px  n_px  phi_x  n_x
%_________________________________________________________________________

% Default values
    if nargin == 0
        n            =    300;
        np           =    2;
        nmax         =    1000;
        alpha        =    1.0;
        rdm          =    0.1;
        speed        =    1;
        betamin      =    0.0;
        gamma        =    10.0;
        m            =    2.0;
        c1           =    2.0;
        c2           =    5.0;
        f_opt_min    =   -0.7;
        f_opt_max    =    0.7;
        teta_min     =    0;
        teta_max     =    pi/2;
        b_min        =    300.0;
        b_max        =    500.0;
        h_min        =    300.0;
        h_max        =    900.0;
        phi_py_min   =    10.0;
        phi_py_max   =    32.0;
        n_py_min     =     0;
        n_py_max     =     0;
        phi_y_min    =    10.0;
        phi_y_max    =    32.0;
        n_y_min      =     0;
        n_y_max      =     0;
        phi_px_min   =    10.0;
        phi_px_max   =    32.0;
        n_px_min     =     2;
        n_px_max     =    10;
        phi_x_min    =    10.0;
        phi_x_max    =    32.0;
        n_x_min      =     2;
        n_x_max      =    10;
        steelcost    =    1.20*7850.0*1e-6;
        concretecost =    100.0*1e-6;
        ksh          =    1;
        shutteringcost =  30.0*1e-3;
        fc           =    30.0;
        fy           =    500.0;
        nd           =    0.0;
        mdx          =    400.0;
        mdy          =    0.0;
        dpx          =    50.0;
        dpy          =    50.0;
        design_code  =    2;
        spiral       =    1;
        lsrd         =    1;
        gsrd         =    1;
        arm_min      =    1;
        duct         =    1;
        sim_x        =    0;
        sim_y        =    0;
        sim_phi      =    0;
        info         =    1;
    end
    
% Check consistency of inputs    
    if or(alpha < 0, alpha > 0.01*n)
        error('incorrectly defined <alpha>');
    elseif or(rdm < 0, rdm > 1)
        error('incorrectly defined <rdm>');
    elseif and(speed ~= 0, speed ~= 1)             
        error('incorrectly defined <speed>');
    elseif or(betamin < 0, betamin > 1)
        error('incorrectly defined <betamin>');
    elseif gamma < 0
        error('incorrectly defined <gamma>');
    elseif m <= 1
        error('incorrectly defined <m>');
    elseif or(c1 < 1, c1 > 2)
        error('incorrectly defined <c1>');
    elseif or(c2 < 5, c2 > 50)
        error('incorrectly defined <c2>');
    elseif or(f_opt_min <= -1, f_opt_min > 0)
        error('incorrectly defined <f_opt_min>');
    elseif or(f_opt_max < 0, f_opt_max >= 1)
        error('incorrectly defined <f_opt_max>');
    elseif f_opt_max < f_opt_min
        error('incorrectly defined <f_opt_min> and/or <f_opt_max>');        
    elseif or(teta_min < 0, teta_min > pi/2)
        error('incorrectly defined <teta_min>');
    elseif or(teta_max <= 0, teta_max > pi/2)
        error('incorrectly defined <teta_max>');
    elseif teta_max < teta_min 
        error('incorrectly defined <teta_min> and/or <teta_max>');
    elseif b_max < b_min 
        error('incorrectly defined <b_min> and/or <b_max>');
    elseif h_max < h_min 
        error('incorrectly defined <h_min> and/or <h_max>');
    elseif or(phi_py_min < 6, phi_py_min > 40)
        error('incorrectly defined <phi_py_min>');
    elseif or(phi_py_max < 6, phi_py_max > 40)
        error('incorrectly defined <phi_py_max>');
    elseif phi_py_max < phi_py_min
        error('incorrectly defined <phi_py_min> and/or <phi_py_max>');
    elseif n_py_min < 0  
        error('incorrectly defined <n_py_min>');
    elseif n_py_max < 0
        error('incorrectly defined <n_py_max>');
    elseif n_py_max < n_py_min
        error('incorrectly defined <n_py_min> and/or <n_py_max>');
    elseif or(phi_y_min < 6, phi_y_min > 40)
        error('incorrectly defined <phi_y_min>');
    elseif or(phi_y_max < 6, phi_y_max > 40)
        error('incorrectly defined <phi_y_max>');
    elseif n_y_min < 0
        error('incorrectly defined <n_y_min>');
    elseif n_y_max < 0
        error('incorrectly defined <n_y_max>');
    elseif n_y_max < n_y_min
        error('incorrectly defined <n_y_min> and/or <n_y_max>');
    elseif or(phi_px_min < 6, phi_px_min > 40)
        error('incorrectly defined <phi_px_min>');
    elseif or(phi_px_max < 6, phi_px_max > 40)
        error('incorrectly defined <phi_px_max>');
    elseif n_px_min < 2
        error('incorrectly defined <n_px_min>');
    elseif n_px_max < 2
        error('incorrectly defined <n_px_max>');
    elseif n_px_max < n_px_min
        error('incorrectly defined <n_px_min> and/or <n_px_max>'); 
    elseif or(phi_x_min < 6, phi_x_min > 40)
        error('incorrectly defined <phi_x_min>');
    elseif or(phi_x_max < 6, phi_x_max > 40)
        error('incorrectly defined <phi_x_max>');
    elseif n_x_min < 2
        error('incorrectly defined <n_x_min>');
    elseif n_x_max < 2
        error('incorrectly defined <n_x_max>');
    elseif n_x_max < n_x_min
        error('incorrectly defined <n_x_min> and/or <n_x_max>'); 
    elseif and(ksh ~= 1, ksh ~= 2)
        error('incorrectly defined <ksh>');
    elseif mdx < mdy
        error(' <mdx> must be greater than <mdy> ');    
    elseif mdx < 0
        error(' <mdx> must be positive');    
    elseif mdy < 0
        error(' <mdy> must be positive');    
    elseif and(design_code ~= 1, design_code ~= 2)
        error('incorrectly defined <design_code>');
    elseif and(spiral ~= 0, spiral ~= 1)
        error('incorrectly defined <spiral>');
    elseif and(lsrd ~= 0, lsrd ~= 1)
        error('incorrectly defined <lsrd>');
    elseif and(gsrd ~= 0, gsrd ~= 1)
        error('incorrectly defined <gsrd>');
    elseif and(arm_min ~= 0, arm_min ~= 1)
        error('incorrectly defined <arm_min>');
    elseif and(duct ~= 0, duct ~= 1)
        error('incorrectly defined <duct>');
    elseif and(sim_x ~= 0, sim_x ~= 1)
        error('incorrectly defined <sim_x>');
    elseif and(sim_y ~= 0, sim_y ~= 1)
        error('incorrectly defined <sim_y>');
    elseif and(sim_phi ~= 0, sim_phi ~= 1)
        error('incorrectly defined <sim_phi>');        
    elseif and(info ~= 0, info ~= 1)
        error('incorrectly defined <info>');
    end
    
    if sim_x*sim_y*sim_phi == 1
        if info
            disp(' Case 1 ')
            disp(' seven variables ')
        end
    elseif sim_x*sim_y*(~sim_phi) == 1
        if info
            disp(' Case 2 ')
            disp(' eight variables ')
        end
    elseif ~sim_x*~sim_y*(~sim_phi) == 1
        if info
            disp(' Case 3 ')
            disp(' twelve variables ')
        end
    else
        error('incorrectly defined inputs: <sim_x> <sim_y> <sim_phi>');
    end
    Cases =[sim_x sim_y sim_phi];
    if sim_x
        phi_px_min=phi_x_min;
        n_px_min=n_x_min;
        phi_px_max=phi_x_max;
        n_px_max=n_x_max;
    end
    if sim_y
        phi_py_min=phi_y_min;
        n_py_min=n_y_min;
        phi_py_max=phi_y_max;
        n_py_max=n_y_max;
    end

% bounds/limits
% Maximun values for design variables from 2 to 12           Variable num.
    Max = [teta_max                                      ... %  2
           b_max            h_max                        ... %  3  4
           phi_py_max       n_py_max                     ... %  5  6
           phi_y_max        n_y_max                      ... %  7  8
           phi_px_max       n_px_max                     ... %  9 10
           phi_x_max        n_x_max];                        % 11 12
    
% Lower/Upper bounds for optimization variables
    if and(n_y_max ~= 0 , n_py_max ~= 0)
        Lb = [f_opt_min                                  ... %  1
              teta_min/teta_max                          ... %  2
              b_min/b_max            h_min/h_max         ... %  3  4
              phi_py_min/phi_py_max  n_py_min/n_py_max   ... %  5  6
              phi_y_min/phi_y_max    n_y_min/n_y_max     ... %  7  8
              phi_px_min/phi_px_max  n_px_min/n_px_max   ... %  9 10
              phi_x_min/phi_x_max    n_x_min/n_x_max];       % 11 12
        Ub =  [ f_opt_max                                ... %  1
                1                                        ... %  2 
                1                 1                      ... %  3  4
                1                 1                      ... %  5  6
                1                 1                      ... %  7  8
                1                 1                      ... %  9 10
                1                 1];                        % 11 12
    elseif and(n_y_max ~= 0 , n_py_max == 0)
        Max(5)=1;
        Lb = [f_opt_min                                  ... %  1
              teta_min/teta_max                          ... %  2
              b_min/b_max            h_min/h_max         ... %  3  4
              phi_py_min/phi_py_max  0                   ... %  5  6*
              phi_y_min/phi_y_max    n_y_min/n_y_max     ... %  7  8
              phi_px_min/phi_px_max  n_px_min/n_px_max   ... %  9 10
              phi_x_min/phi_x_max    n_x_min/n_x_max];       % 11 12
        Ub =  [ f_opt_max                                ... %  1
                1                                        ... %  2 
                1                 1                      ... %  3  4
                1                 0                      ... %  5  6*
                1                 1                      ... %  7  8
                1                 1                      ... %  9 10
                1                 1];                        % 11 12        
    elseif and(n_y_max == 0 , n_py_max ~= 0)
        Max(7)=1;
        Lb = [f_opt_min                                  ... %  1
              teta_min/teta_max                          ... %  2
              b_min/b_max            h_min/h_max         ... %  3  4
              phi_py_min/phi_py_max  n_py_min/n_py_max   ... %  5  6
              phi_y_min/phi_y_max    0                   ... %  7  8*
              phi_px_min/phi_px_max  n_px_min/n_px_max   ... %  9 10
              phi_x_min/phi_x_max    n_x_min/n_x_max];       % 11 12
        Ub =  [ f_opt_max                                ... %  1
                1                                        ... %  2 
                1                 1                      ... %  3  4
                1                 1                      ... %  5  6
                1                 0                      ... %  7  8*
                1                 1                      ... %  9 10
                1                 1];                        % 11 12                    
    elseif and(n_y_max == 0 , n_py_max == 0)
        Max(5)=1;
        Max(7)=1;
        Lb = [f_opt_min                                  ... %  1
              teta_min/teta_max                          ... %  2
              b_min/b_max            h_min/h_max         ... %  3  4
              phi_py_min/phi_py_max  0                   ... %  5  6*
              phi_y_min/phi_y_max    0                   ... %  7  8*
              phi_px_min/phi_px_max  n_px_min/n_px_max   ... %  9 10
              phi_x_min/phi_x_max    n_x_min/n_x_max];       % 11 12
        Ub =  [ f_opt_max                                ... %  1
                1                                        ... %  2 
                1                 1                      ... %  3  4
                1                 0                      ... %  5  6*
                1                 0                      ... %  7  8*
                1                 1                      ... %  9 10
                1                 1];                        % 11 12                            
    end
    if all(Cases)
        Max(8:10)=[];
        Max(4:5)=[];
        Max(4)=max([phi_y_max phi_x_max]);
        Max(5)=n_y_max;
        if Max(5) == 0, Max(5)=1; end
        Max(6)=n_x_max;
        Lb(9:11)=[];
        Lb(5:6)=[];
        Lb(5)=phi_x_min./Max(4);
        Lb(6)=n_y_min./Max(5);
        Ub(9:11)=[];
        Ub(5:6)=[];
    elseif Cases(1) && Cases(2)
        Max(8:9)=[];
        Max(4:5)=[];
        Max(4)=phi_y_max;
        Max(5)=n_y_max;
        if Max(5) == 0, Max(5)=1; end
        Max(6)=phi_x_max;
        Max(7)=n_x_max;
        Lb(9:10)=[];
        Lb(5:6)=[];
        Lb(5)=phi_y_min./Max(4);
        Lb(6)=n_y_min./Max(5);
        Lb(7)=phi_x_min./Max(6);
        Lb(8)=n_x_min./Max(7);
        Ub(9:10)=[];
        Ub(5:6)=[];        
    end
 
% Maximum value of steel reinforcement
    as_max = n_py_max * (pi / 4) * phi_py_max ^ 2 + ...
              n_y_max * (pi / 4) *  phi_y_max ^ 2 + ...
             n_px_max * (pi / 4) * phi_px_max ^ 2 + ...
              n_x_max * (pi / 4) *  phi_x_max ^ 2;

% Maximum value of cost    
    cost_max_0 = steelcost*as_max+concretecost*(b_max*h_max-as_max)+...
                 shutteringcost*(ksh*b_max+2*h_max);

% Parameters
    Para = [round(n/np) np nmax alpha rdm speed betamin gamma m lsrd gsrd];

% Data for constraints and fitness function
    Data = [steelcost concretecost shutteringcost ksh nd mdx mdy fc fy ...
            cost_max_0 dpx dpy arm_min duct design_code spiral c1 c2 Max]; 

% Name (handle) of the objective function (Cost)
    fhandle = @cost;

% Main function for constrained optimization
    [V_best,cost_best,G_best,n_tot] = ...
                      mpfa_cnstr(fhandle, Lb, Ub, Para, Data, info);
end
