function [N_u,M_ux,M_uy,rot,f_duct,d,h_e,A_s,A_sp,yrLN,XR] = ...
                mpfa_rcsde(n_fiber,f,teta,b,h,phi_y,n_y,phi_py,n_py,...
                                      phi_x,n_x,phi_px,n_px,...
                                      d_px,d_py,f_c,f_y,design_code)
% Strength and properties of the section
%                                              
% Input argument:
%          n_fiber     number of fibers
%          f           depth of neutral axis from extreme compression fiber
%          teta        angle of the neutral fiber
%          b           width of rectangular cross section
%          h           height of rectangular cross section
%          phi_y       diameter of bars at the left side of the section
%          n_y         number of bars at the left side of the section
%          phi_py      diameter of bars at the right side of the section
%          n_py        number of bars at the right side of the section
%          phi_x       diameter of bars at the bottom side of the section
%          n_x         number of bars at the bottom side of the section
%          phi_px      diameter of bars at the top side of the section
%          n_px        number of bars at the top side of the section
%          d_px        clear cover of reinforcement plus stirrup diameter (x-axis)
%          d_py        clear cover of reinforcement plus stirrup diameter (y-axis)
%          f_c         strength of concrete (MPa o N/mm2)
%          f_y         strength of steel bars (MPa o N/mm2)
%          design_code 1: EC2; 2: ACI318
%
% Output arguments:
%          N_u         axial strength
%          M_ux        moment strength about x-axis
%          M_uy        moment strength about y-axis
%          rot         section gyration (mm-1)
%          f_duct      maximum neutral axis depth
%          d           effective height (y*-axis direction)
%          h_e         height (y*-axis direction)
%          A_s         total area of reinforcement bars in tension
%          A_sp        total area of reinforcement bars in compression
%
% Number of corner points, axes, distances and neutral axis (NA)
%                                    
%                    y  ^
%                       |
%                       |
%                       ^  M_y
%                       ^
%             NA        |
%              \        |
%               \       |       
%             4 -\------|-------- 1 ----------
%               | \     |       |      |     |
%               |  \0   0   0   |      |     |
%               |   \   |       |      |     |
%               |   o\  |   .   |      |     |
%        M_x    |     \ |       |      |     |
%   x <---<<--------o--\--  .   |  d_y |   h |
%               |       \       |      |     |
%               |   o    \  .   |      |     |
%               |         \     |      |     |
%               |   O   O  \O   |      -     |
%               |           \   | d_py |     |
%             3 -------------\--- 2 ----------
%               |d_px    d_x  \ |             
%               |---|-----------|
%               |       b       |
%               |---------------|
%
%
% Number of the points considered for steel bars and spacing between them
%                        
%                   s_px s_px
%                   |---|---|   
%                               
%               -----------------    
%               |  10  11  12   |            
%       -       |   0   0   0   |        -     
%   s_y |       |               |   s_py |     
%       -       | 9 o       . 1 |        -     
%   s_y |       |               |   s_py |     
%       -       | 8 o       . 2 |        -    
%   s_y |       |               |   s_py |     
%       -       | 7 o       . 3 |        -     
%   s_y |       |               |   s_py |     
%       -       |   O   O   O   |        -     
%               |   6   5   4   |            
%               -----------------    
%
%                    s_x s_x                  
%                   |---|---|    
%
%
% Positive values for stresses in compression
%
% Units: N, mm, mm2, MPa                                             
% _________________________________________________________________________

    if (n_y  - fix(n_y))  ~= 0, n_y  =  fix(n_y);   end
    if (n_py - fix(n_py)) ~= 0, n_py =  fix(n_py);  end
    if (n_x  - fix(n_x))  ~= 0, n_x  =  fix(n_x);   end
    if (n_px - fix(n_px)) ~= 0, n_px =  fix(n_px);  end

    d_px   = d_px + 0.5 * max([phi_py phi_y phi_px phi_x]);
    d_py   = d_py + 0.5 * max([phi_px phi_x]);

% Points A and B where the NA intersects sides of the section
    X1     = [-b/2  h/2]';
    X2     = [-b/2 -h/2]';
    X3     = [ b/2 -h/2]';
    X4     = [ b/2  h/2]';
    R      = [cos(teta) sin(teta); -sin(teta) cos(teta)];
    Xr1    = R * X1;
    Xr2    = R * X2;
    Xr3    = R * X3;
    Xr4    = R * X4;
    XR     = [Xr1 Xr2 Xr3 Xr4];
    xrI    = (-b / 2) * cos(teta) + (h / 2) * sin(teta) + f / tan(teta);
    xrII   =  (b / 2) * (2 - cos(teta) ^ 2) / cos(teta) + ...
              (h / 2) * sin(teta) - f * tan(teta);
    xrIII  = (-b / 2) * cos(teta) - ...
              (h / 2) * (2 - sin(teta) ^ 2) / sin(teta) + f / tan(teta);
    xrIV   = (-b/2)*cos(teta) + (h/2)*sin(teta) - f*tan(teta);
    COORXr = [xrI xrII xrIII xrIV];
    yrLN   = (h / 2) * cos(teta) + (b / 2) * sin(teta) - f;
    [Xr]   = sort(COORXr,2,'descend');
    Yr     = [yrLN yrLN yrLN yrLN];
    xrA    = Xr(2);
    yrA    = Yr(2);
    xrB    = Xr(3);
    yrB    = Yr(3);
    XrA    = [xrA yrA]';
    XrB    = [xrB yrB]';
    XA     = R' * XrA;
    xA     = XA(1);
    yA     = XA(2);
    XB     = R' * XrB;
    xB     = XB(1);
    yB     = XB(2);
    if teta == 0
        if f < (h/2)
            xA =  b / 2;
            yA = (h / 2) - f;
            xB = -b / 2;
            yB = (h / 2) - f;
        else
            xA =  b / 2;
            yA = -f + (h / 2);
            xB = -b / 2;
            yB = -f + (h / 2);            
        end
    end
    if teta == pi/2
        xA  = -(b / 2) + f;
        yA  =   h / 2;
        xB  = -(b / 2) + f;
        yB  =  -h / 2;
    end

% Points for reinforcement bars
    n_points = n_py + n_x + n_y + n_px;
    X_points = zeros(n_points,2);

% Spacing between points
    s_px = (b - 2 * d_px) / (n_px - 1);
    s_x  = (b - 2 * d_px) / (n_x - 1);
    s_py = (h - 2 * d_py) / (n_py + 1);
    s_y  = (h - 2 * d_py) / (n_y + 1);

% Points at the right side
    for iloop = 1 : n_py
        X_points(iloop,1) = -b / 2 + d_px;
        X_points(iloop,2) =  h / 2 - d_py - iloop * s_py;
    end

% Points at the bottom side
    for iloop = 1 : n_x
        X_points(iloop+n_py,1) = -b / 2 + d_px + (iloop-1) * s_x;
        X_points(iloop+n_py,2) = -h / 2 + d_py;
    end

% Points at the left side
    for iloop = 1 : n_y
        X_points(iloop+n_py+n_x,1) =  b / 2 - d_px;
        X_points(iloop+n_py+n_x,2) = -h / 2 + d_py + iloop * s_y;
    end

% Points at the top side
    for iloop = 1 : n_px
        X_points(iloop+n_py+n_x+n_y,1) = b / 2 - d_px - (iloop-1) * s_px;
        X_points(iloop+n_py+n_x+n_y,2) = h / 2 - d_py;
    end

% Distances
    DIST = zeros(n_points,1);
    for iloop=1 : n_points
        xP            = X_points(iloop,1);
        yP            = X_points(iloop,2);
        DIST(iloop,1) = ((xA - xB) * (yP - yB) - (yA - yB) * (xP - xB))/...
                        sqrt((xA - xB) ^ 2 + (yA - yB) ^ 2);    
    end

% Obtaining d and h_e
    max_y = 0;
    for iloop=1 : n_points
        XP  = X_points(iloop,:)';
        XrP = R * XP;
        if XrP(2) < 0
            max_y = max(max_y,-XrP(2));
        end
    end
    Xr1 = R * X1;
    d   = Xr1(2) + max_y;
    Xr3 = R * X3;
    h_e = Xr1(2) - Xr3(2);

% Strains
    f_ck  = f_c;
    f_yk  = f_y;
    E_s   = 2e5;
    if design_code == 1    % EC2
        gamma_s = 1.15; 
        f_yd    = f_yk / gamma_s;
        eps_sy  = f_yd / E_s;
        eps_su  = 0.01;    
        gamma_c = 1.5;
        f_cd    = f_ck / gamma_c;
        if f_ck < 50
            eps_cu = 0.0035;
            eps_c  = 0.002;
            f_lim  = d / (1 + 1.429e-3 * f_yd);
            f_duct = eps_cu * d / (eps_su + eps_cu);
        elseif f_ck <= 90
            eps_cu = (2.6 + 35 * ((90 - f_ck) / 100) ^ 4) / 1000;
            eps_c  = (2 + 0.085 * (f_ck - 50) ^ 0.53) / 1000;
            f_lim  = eps_cu * d / (eps_sy + eps_cu); 
            f_duct = eps_cu * d / (eps_su + eps_cu);
        else
            disp('f_ck must be less or equal than 90 MPa')
        end
    elseif design_code == 2   % ACI318
        f_yd = f_yk;
        if f_yd > 420 
            eps_sy = 0.002;
        else
            eps_sy = f_yd / E_s;
        end
        eps_su = 0.005;        
        eps_cu = 0.003;
        eps_c  = 0.002;
        f_lim  = eps_cu * d / (eps_sy + eps_cu);
        f_duct = eps_cu * d / (eps_su + eps_cu);
        f_cd   = f_ck;    
    end

    if f < 0                                    % domain 1-2
        dist = d - f;
        epsilon = eps_su;
    elseif f < (eps_cu * d / (eps_su + eps_cu)) % domain 2-3
        dist = d - f;
        epsilon = eps_su;
    elseif f < f_lim                            % domain 3-4
        dist = f;
        epsilon = eps_cu;
    elseif f < d                                % domain 4-4'
        dist = f;
        epsilon = eps_cu;
    elseif f < h_e                              % domain 4'-5
        dist = f;
        epsilon = eps_cu;
    else                                        % domain 5-6
        dist=f - (1 - eps_c / eps_cu) * h_e;
        epsilon = eps_c;
    end
    rot  = epsilon / dist;
    Xr_points=zeros(size(X_points));
    for iloop=1 : n_points
        XP    = X_points(iloop,:)';
        XrP   = R * XP;
        Xr_points(iloop,1) = XrP(1);
        Xr_points(iloop,2) = XrP(2);
    end
    EPS_s=zeros(n_points,1);
    for iloop=1 : n_points
        EPS_s(iloop,1) = (epsilon/dist) * ...
                         ((h_e/2) - f - Xr_points(iloop,2));
    end    

% Stresses and forces
    N_u   = 0;
    M_srx = 0;
    M_sry = 0;
    A_s   = 0;
    A_sp  = 0;
    for iloop = 1 : n_points
        eps = EPS_s(iloop,1);
        eps2 = eps;
        if abs(eps) > eps_sy
            eps = sign(eps) * eps_sy;
        end
        sigma=E_s * eps;
        if iloop <= n_py
            area = (pi / 4) * phi_py ^ 2;
        elseif  iloop <= n_py + n_x
            area = (pi / 4) * phi_x ^ 2;
        elseif  iloop <= n_py + n_x + n_y
            area = (pi / 4) * phi_y ^ 2;    
        else
            area = (pi / 4) * phi_px ^ 2;    
        end
        if eps > 0
            A_s = A_s + area;
        else
            A_sp = A_sp + area;
        end
        if eps2 < -eps_c
            sigma_c = -f_cd;
        elseif eps2 < 0
            if f_ck < 50
                n = 2;
            elseif f_ck <= 90
                n = 1.4 + 23.4 * ((90 - f_ck) / 100) ^ 4;
            else
                n = 1.4;
            end
            sigma_c = -f_cd * (1 - (1 - (abs(eps2) / eps_c)) ^ n);
        else
            sigma_c = 0;
        end
        M_srx = M_srx - (sigma-sigma_c) * area * Xr_points(iloop,2);
        M_sry = M_sry + (sigma-sigma_c) * area * Xr_points(iloop,1);
        N_u   = N_u   + (sigma-sigma_c) * area;
    end

    if f > 0
        if f < d
            Nloop = ceil((f/d) * n_fiber);
        else
            Nloop = n_fiber;
        end
        delta_yr = f / Nloop;
        y_r = yrLN;
        M_crx = 0;
        M_cry = 0;
        for jloop = 1 : Nloop
            y_r     = y_r + delta_yr/2;
            eps2   = (epsilon / dist) * ((h_e / 2) - f - y_r);
            if eps2 < - eps_c
                sigma_c = -f_cd;
            elseif eps2 < 0
                if f_ck < 50
                    n = 2;
                elseif f_ck <= 90
                    n = 1.4 + 23.4 * ((90 - f_ck) / 100) ^ 4;
                else
                    n = 1.4;
                end
                sigma_c = -f_cd * (1 - (1 - (abs(eps2) / eps_c)) ^ n);
            else
                sigma_c = 0;
            end
            if teta > 0
                xrI_2    = (-b / 2) * cos(teta) + (h / 2) * sin(teta) + ...
                           (f - (2*jloop - 1) * delta_yr / 2) / tan(teta);
                xrII_2   = (b / 2) * (2 - cos(teta) ^ 2) / cos(teta) + ...
                           (h / 2) * sin(teta) - ...
                           (f - (2 *jloop - 1) * delta_yr / 2) * tan(teta);
                xrIII_2  = (-b / 2) * cos(teta) - ...
                           (h / 2) * (2 - sin(teta) ^ 2) / sin(teta) + ...
                           (f - (2 *jloop - 1) * delta_yr / 2) / tan(teta);
                xrIV_2   = (-b / 2) * cos(teta) + (h / 2) * sin(teta) - ...
                           (f - (2 *jloop - 1) * delta_yr / 2) * tan(teta);
                COORXr_2 = [xrI_2 xrII_2 xrIII_2 xrIV_2];
                [Xr_2]   = sort(COORXr_2,2,'descend');    
                b_r      = Xr_2(2) - Xr_2(3);
                arm_x    = y_r;
                arm_y    = (Xr_2(2) + Xr_2(3)) / 2;
            else
                b_r    = b;
                arm_x  = y_r;
                arm_y  = 0;            
            end            
            M_crx    = M_crx - sigma_c * b_r * delta_yr * arm_x;
            M_cry    = M_cry + sigma_c * b_r * delta_yr * arm_y;
            N_u      = N_u   + sigma_c * b_r * delta_yr;
            y_r      = y_r + delta_yr / 2;
        end
    else
        M_crx = 0;
        M_cry = 0;
    end
 
    M_c  = R'*[M_crx M_cry]';
    M_cx = M_c(1);
    M_cy = M_c(2);
    M_s  = R'*[M_srx M_sry]';
    M_sx = M_s(1);
    M_sy = M_s(2);
    M_ux = M_cx + M_sx;
    M_uy = M_cy + M_sy;
    N_u  = -N_u;
end

